@@ -1,6 +1,6 @@
 /* -*- c-file-style: "ruby"; indent-tabs-mode: nil -*- */
 /*
- *  Copyright (C) 2011-2015  Ruby-GNOME2 Project Team
+ *  Copyright (C) 2011-2019  Ruby-GNOME2 Project Team
  *  Copyright (C) 2002,2003  Masahiro Sakai
  *
  *  This library is free software; you can redistribute it and/or
@@ -220,7 +220,7 @@ rbg_cstr2rval_with_free(gchar *str)
     return rbg_cstr2rval_free(str);
 }
 
-static rb_encoding *filename_encoding_if_not_utf8;
+rb_encoding *rbg_filename_encoding;
 
 static VALUE
 rbg_filename_to_ruby_body(VALUE filename)
@@ -234,9 +234,9 @@ rbg_filename_to_ruby_body(VALUE filename)
 
     /* if needed, change encoding of Ruby String to filename encoding, so that
        upcoming File operations will work properly */
-    return filename_encoding_if_not_utf8 != NULL ?
-        rb_str_export_to_enc(rb_filename, filename_encoding_if_not_utf8) :
-        rb_filename;
+    return rbg_filename_encoding == rb_utf8_encoding() ?
+        rb_filename :
+        rb_str_export_to_enc(rb_filename, rbg_filename_encoding);
 }
 
 static VALUE
@@ -257,7 +257,7 @@ rbg_filename_to_ruby(const gchar *filename)
     if (filename == NULL)
         return Qnil;
 
-    if (filename_encoding_if_not_utf8 == NULL)
+    if (rbg_filename_encoding == rb_utf8_encoding())
         return CSTR2RVAL(filename);
 
     filename_utf8 = g_filename_to_utf8(filename, -1, NULL, &written, &error);
@@ -278,15 +278,15 @@ rbg_filename_to_ruby_free(gchar *filename)
         return Qnil;
 
     /* convert filename to UTF-8 if needed */
-    if (filename_encoding_if_not_utf8 != NULL) {
+    if (rbg_filename_encoding == rb_utf8_encoding()) {
+        filename_utf8 = filename;
+    } else {
         GError *error = NULL;
 
         filename_utf8 = g_filename_to_utf8(filename, -1, NULL, &written, &error);
         g_free(filename);
         if (error != NULL)
             RAISE_GERROR(error);
-    } else {
-        filename_utf8 = filename;
     }
 
     return rb_ensure(rbg_filename_to_ruby_body, (VALUE)filename_utf8,
@@ -306,10 +306,14 @@ rbg_filename_from_ruby(VALUE filename)
         filename = rb_str_export_to_enc(filename, rb_utf8_encoding());
 
     /* convert it to filename encoding if needed */
-    if (filename_encoding_if_not_utf8 == NULL)
-        return g_strdup(RSTRING_PTR(filename));
+    if (rbg_filename_encoding == rb_utf8_encoding())
+        return g_strndup(RSTRING_PTR(filename), RSTRING_LEN(filename));
 
-    retval = g_filename_from_utf8(RSTRING_PTR(filename), -1, NULL, &written, &error);
+    retval = g_filename_from_utf8(RSTRING_PTR(filename),
+                                  RSTRING_LEN(filename),
+                                  NULL,
+                                  &written,
+                                  &error);
     if (error != NULL)
         RAISE_GERROR(error);
 
@@ -1055,9 +1059,9 @@ Init_glib2(void)
     rb_define_const(RG_TARGET_NAMESPACE, "MAXSHORT", INT2FIX(G_MAXSHORT));
     rb_define_const(RG_TARGET_NAMESPACE, "MAXUSHORT", UINT2NUM(G_MAXUSHORT));
 
-    rb_define_const(RG_TARGET_NAMESPACE, "MINLONG", INT2FIX(G_MINLONG));
-    rb_define_const(RG_TARGET_NAMESPACE, "MAXLONG", INT2NUM(G_MAXLONG));
-    rb_define_const(RG_TARGET_NAMESPACE, "MAXULONG", UINT2NUM(G_MAXULONG));
+    rb_define_const(RG_TARGET_NAMESPACE, "MINLONG", LL2NUM(G_MINLONG));
+    rb_define_const(RG_TARGET_NAMESPACE, "MAXLONG", LL2NUM(G_MAXLONG));
+    rb_define_const(RG_TARGET_NAMESPACE, "MAXULONG", ULL2NUM(G_MAXULONG));
 
     rb_define_const(RG_TARGET_NAMESPACE, "MININT8", INT2FIX(G_MININT8));
     rb_define_const(RG_TARGET_NAMESPACE, "MAXINT8", INT2FIX(G_MAXINT8));
@@ -1071,16 +1075,16 @@ Init_glib2(void)
     rb_define_const(RG_TARGET_NAMESPACE, "MAXINT32", INT2NUM(G_MAXINT32));
     rb_define_const(RG_TARGET_NAMESPACE, "MAXUINT32", UINT2NUM(G_MAXUINT32));
 
-    rb_define_const(RG_TARGET_NAMESPACE, "MININT64", INT2FIX(G_MININT64));
+    rb_define_const(RG_TARGET_NAMESPACE, "MININT64", LL2NUM(G_MININT64));
     rb_define_const(RG_TARGET_NAMESPACE, "MAXINT64", LL2NUM(G_MAXINT64));
     rb_define_const(RG_TARGET_NAMESPACE, "MAXUINT64", ULL2NUM(G_MAXUINT64));
 
-    rb_define_const(RG_TARGET_NAMESPACE, "MAXSIZE", UINT2NUM(G_MAXSIZE));
+    rb_define_const(RG_TARGET_NAMESPACE, "MAXSIZE", ULL2NUM(G_MAXSIZE));
 
-    rb_define_const(RG_TARGET_NAMESPACE, "MINFLOAT", INT2FIX(G_MINFLOAT));
+    rb_define_const(RG_TARGET_NAMESPACE, "MINFLOAT", DBL2NUM(G_MINFLOAT));
     rb_define_const(RG_TARGET_NAMESPACE, "MAXFLOAT", DBL2NUM(G_MAXFLOAT));
 
-    rb_define_const(RG_TARGET_NAMESPACE, "MINDOUBLE", INT2FIX(G_MINDOUBLE));
+    rb_define_const(RG_TARGET_NAMESPACE, "MINDOUBLE", DBL2NUM(G_MINDOUBLE));
     rb_define_const(RG_TARGET_NAMESPACE, "MAXDOUBLE", DBL2NUM(G_MAXDOUBLE));
 
     /* Standard Macros */
@@ -1097,12 +1101,13 @@ Init_glib2(void)
         || filename_charsets[0] == NULL
         || !strcmp(filename_charsets[0], "UTF-8")
         || rb_enc_find(filename_charsets[0]) == rb_enc_find("ASCII-8BIT")) {
-        /* set to NULL, mean do not perform transcoding, either filename
-           encoding is unknown, UTF-8, or unsupported */
-        filename_encoding_if_not_utf8 = NULL;
+        rbg_filename_encoding = rb_utf8_encoding();
     } else {
-        filename_encoding_if_not_utf8 = rb_enc_find(filename_charsets[0]);
+        rbg_filename_encoding = rb_enc_find(filename_charsets[0]);
     }
+    rb_define_const(RG_TARGET_NAMESPACE,
+                    "FILENAME_ENCODING",
+                    rb_enc_from_encoding(rbg_filename_encoding));
 
 /* Don't implement them.
 #define     G_DIR_SEPARATOR_S
@@ -1146,6 +1151,9 @@ union       GDoubleIEEE754;
     rb_define_const(RG_TARGET_NAMESPACE, "PRIORITY_LOW", INT2FIX(G_PRIORITY_LOW));
 
 /*    Init_mem(); */
+
+    Init_glib_gc();
+
     Init_gutil();
     Init_gutil_callback();
 
@@ -1162,6 +1170,7 @@ union       GDoubleIEEE754;
     Init_gobject();
 
     /* Require GBoxed/GObject */
+    Init_glib_error_conversions();
     Init_glib_utils();
     Init_glib_spawn();
     Init_glib_spawnerror();
@@ -1189,4 +1198,5 @@ union       GDoubleIEEE754;
     Init_glib_matchinfo();
     Init_glib_date_time();
     Init_glib_time_zone();
+    Init_glib_bytes();
 }
